"""Genai: generative AI tooling for IPython and Jupyter notebooks

## Magic methods

    - `%%assist`: Assist you in writing code.

## Usage

  `%load_ext genai` to load the extension in other notebook
  projects
"""

__version__ = "0.10.0"


def load_ipython_extension(ipython):
    from .suggestions import register
    from .magics import assist

    register()


def unload_ipython_extension(ipython):
    # Unload the custom exception handler
    ipython.set_custom_exc((Exception,), None)



import openai

NOTEBOOK_CODING_ASSISTANT_TEMPLATE = """You are a notebook coding assistant,
designed to help users write their next code cell. The primary language is python.
Write comments and code for the user to read and run. The user will be able to
edit the code and run it.
"""


def generate_next_cell(
    context,  # List[Dict[str, str]]
    text,
):
    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo",
        messages=[
            # Establish the context in which GPT will respond
            {
                "role": "system",
                "content": NOTEBOOK_CODING_ASSISTANT_TEMPLATE,
            },
            # In, Out
            *context,
            # The user code/text
            {
                "role": "user",
                "content": text,
            },
        ],
    )

    text = completion["choices"][0]["message"]["content"]

    return text



"""Magic to generate code cells for notebooks using OpenAI's API."""

from IPython import get_ipython
from IPython.core.magic import register_cell_magic
from IPython.core.magic_arguments import argument, magic_arguments, parse_argstring
from IPython.display import display, HTML

from genai.components import completion_made, starting_message
from genai.context import get_historical_context
from genai.generate import generate_next_cell


@magic_arguments()
@argument("--fresh", action="store_true")
@argument(
    "--verbose",
    action="store_true",
    help="Show a full log in the cell output",
)
@argument(
    "--in-place",
    action="store_true",
    help="Replace the current cell with the generated code",
)
@register_cell_magic
def assist(line, cell):
    progress = display(HTML(starting_message()), display_id=True)

    args = parse_argstring(assist, line)

    ip = get_ipython()
    cell_text = cell.strip()

    context = []
    if not args.fresh:
        context = get_historical_context(ip)

    if args.verbose:
        display(
            f"""magic arguments: {line}
submission: {cell}"""
        )

    generated_text = generate_next_cell(context, cell_text)

    progress.update(HTML(completion_made()))

    new_cell = f"""{cell_text}\n{generated_text}"""

    if args.in_place:
        new_cell = f"""#%%assist {line}\n{cell_text}\n{generated_text}"""

    ip.set_next_input(
        new_cell,
        replace=args.in_place,
    )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['genai']

package_data = \
{'': ['*']}

install_requires = \
['ipython>=8.10.0,<9.0.0', 'openai>=0.27.0,<0.28.0', 'vdom>=0.6,<0.7']

setup_kwargs = {
    'name': 'genai',
    'version': '0.10.0',
    'description': 'Generative AI for IPython (enhance your code cells)',
    'long_description': None,
    'author': 'Kyle Kelley',
    'author_email': 'rgbkrk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
